@@ -15,11 +15,12 @@ Example of valid vimoku.ini:
 
 """
 
-__version__ = '1.0.4'
+__version__ = '1.0.5'
 
 
 
 import os
+import re
 import time
 import shlex
 import shutil
@@ -28,12 +29,18 @@ import argparse
 import subprocess
 import configparser
 from collections import defaultdict
-from dokuwikixmlrpc import DokuWikiClient, DokuWikiXMLRPCError
+try:
+    from dokuwikixmlrpc import DokuWikiClient, DokuWikiXMLRPCError
+except ImportError:
+    print('Python package dokuwikixmlrpc not installed.')
+    print('You may want to run something like `pip3 install dokuwikixmlrpc --user -U`')
+    exit(1)
 
 
 DEFAULT_EDITOR = 'vi'  # used if CLI, configfile and $EDITOR are empty.
 DEFAULT_MESSAGE = 'undocumented remote modification'
-DEBUG = True  # set to true to get many lines of logging
+DEFAULT_SUPPRESSION_MESSAGE = 'page deleted'
+DEBUG = False  # set to true to get many lines of logging
 DRY_RUN = False  # set to true to prevent any page modification
 REDIRECTION = 'This page has been moved [[{newname}|here]].'
 TERM_WIDTH = shutil.get_terminal_size().columns
@@ -171,11 +178,17 @@ def get_client_from_parsed_config(config:configparser.ConfigParser) -> DokuWikiC
         return theclient
     return with_additional_methods(client)
 
-def get_editor_and_options(configfile:str):
+def get_editor_command(configfile:str) -> str:
     config = read_config(configfile)
-    editor_options = config['options']['editor_options']
     editor = config['options']['editor'] or os.environ.get('EDITOR') or DEFAULT_EDITOR
-    return editor, editor_options
+    return editor
+
+def get_aliases(configfile:str) -> str:
+    config = read_config(configfile)
+    alias_string = config['options'].get('aliases', '')
+    aliases = []
+    REGEX_ALIAS = re.compile('([a-zA-Z0-9:._%-]+):\s+([a-zA-Z0-9:._%-]+)')
+    return dict(REGEX_ALIAS.findall(alias_string))
 
 def try_lock(allpagenames, clients):
     if isinstance(allpagenames, str): allpagenames = [allpagenames]
@@ -223,23 +236,26 @@ def create_unique_dir(config) -> str:
     os.mkdir(edition_dir)
     return edition_dir
 
-def run_editor(editor, editor_options, filenames):
+def run_editor(editor_cmd, filenames):
     """Run editor properly. Finishes when user finishes."""
+    filenames = tuple(filenames)
     # print('FILES TO EDIT:', ', '.join(filenames))
-    comdir = os.path.commonpath(tuple(filenames))
+    comdir = os.path.commonpath(filenames)
     if not os.path.isdir(comdir):  # isfile would return False if there is only 1 filename, that is a new file not yet created on the system
         comdir = os.path.dirname(comdir)
-    options = shlex.split(editor_options.format(cwd=comdir))
-    command = [editor, *options, *filenames]
+    command = shlex.split(editor_cmd.format(pages=shlex.join(filenames), cwd=comdir))
+    if '{pages}' not in editor_cmd:  # add the page list at the end
+        command.extend(filenames)
     lprint('Invoking editor with:', command)
     p = subprocess.Popen(command)
     p.wait()
 
 
-def setdict_sequence(editor, editor_options, assocs:dict, objname='lines', action='remove from choices'):
+def setdict_sequence(editor_cmd, assocs:dict, objname='lines', action='remove from choices'):
     """Provide user with a list of lines. Return those that weren't deleted"""
     # get the best key representation
     def simplified_key(key:str) -> str:
+        # print('SIMPLIFIED_KEY:', key, '\t####\t', editor_cmd)
         if len(set(map(os.path.dirname, assocs.keys()))) == 1:  # only one common prefix
             return os.path.basename(key.strip())
         return key.strip()  # different prefix -> some names could be equal
@@ -254,7 +270,7 @@ def setdict_sequence(editor, editor_options, assocs:dict, objname='lines', actio
             fd.write(f"{revsimplassocs[key].ljust(max_key_len)}: {val}\n")
         fd.write(f"\n\n# lines starting with a '#' will be ignored.\n# edit lines freely, but keep the colons followed by a space.\n# delete {objname} you want to {action}.")
     # run the editor, retrieve the user's choice
-    run_editor(editor, editor_options, [tmpfile])
+    run_editor(editor_cmd, [tmpfile])
     assocs = {}
     with open(tmpfile) as fd:
         for line in fd:
@@ -265,13 +281,13 @@ def setdict_sequence(editor, editor_options, assocs:dict, objname='lines', actio
     return assocs
 
 
-def choice_sequence(editor, editor_options, choices, objname='lines', action='remove from choices'):
+def choice_sequence(editor_cmd, choices, objname='lines', action='remove from choices'):
     """Provide user with a list of lines. Return those that weren't deleted"""
     # create the file to edit, and run the editor
     with tempfile.NamedTemporaryFile('w', delete=False) as fd:
         tmpfile = fd.name
         fd.write('\n'.join(map(str, choices)) + '\n\n' + f"# lines starting with a '#' will be ignored.\n# delete {objname} you want to {action}.")
-    run_editor(editor, editor_options, [tmpfile])
+    run_editor(editor_cmd, [tmpfile])
     # get the user's choices
     kept = []
     with open(tmpfile) as fd:
@@ -284,7 +300,7 @@ def choice_sequence(editor, editor_options, choices, objname='lines', action='re
     return kept
 
 
-def edition_sequence(editor, editor_options, edition_dir, fullpagenames, clients):
+def edition_sequence(editor_cmd, edition_dir, fullpagenames, clients):
     # retrieve each page, put it in the edition directory
     filehashes = {}  # filename -> (page, hash)   (to later determine if a modification was made)
     for fullpagename in fullpagenames:
@@ -300,9 +316,10 @@ def edition_sequence(editor, editor_options, edition_dir, fullpagenames, clients
             # NB: if the file doesn't exist, let the editor create it ; it will indicate the «new file» status to the user, confirming the inexistance of the file on the wiki.
         filehashes[fname] = fullpagename, hash(content)
     # edition
-    run_editor(editor, editor_options, filehashes)
+    run_editor(editor_cmd, filehashes)
     # detect and send the modified files
     modified_files = {}  # filename -> page
+    deleted_page_filenames = set()  # set of filenames which pages will be deleted by edition (i.e. no more content)
     for fname, (fullpagename, ini_hash) in filehashes.items():
         try:
             with open(fname) as fd:  new_hash = hash(fd.read())
@@ -311,7 +328,9 @@ def edition_sequence(editor, editor_options, edition_dir, fullpagenames, clients
         else:
             if new_hash != ini_hash:
                 modified_files[fname] = fullpagename
-    return edition_dir, modified_files, set(filehashes.keys())
+                if new_hash == hash(''):
+                    deleted_page_filenames.add(fname)
+    return edition_dir, modified_files, set(filehashes.keys()), deleted_page_filenames
 
 
 def upload_work(modified_files, messages:dict, clients):
@@ -344,6 +363,19 @@ def edit_pages(pages, message, config, clients, cli_args):
         pages = run_main_sequence(pages, message, config, clients, cli_args)
 
 
+def substitute_aliases(fullpagenames:[str], config) -> [str]:
+    """Return page names, with aliases substituted"""
+    aliases = get_aliases(config)
+    def replace_alias(fullpagename:str) -> str:
+        wiki, pagename = wiki_page_from_fullpagename(fullpagename, '')
+        for alias, value in aliases.items():
+            if pagename.startswith(alias):
+                return pagename.replace(alias, value, 1)
+        return fullpagename_from_wiki_page(wiki, pagename) if wiki else pagename
+    return tuple(map(replace_alias, fullpagenames))
+
+
+
 def run_main_sequence(pages, message, config, clients, cli_args):
     """Lock, retrieve, let user edit, upload and cleanup, the asked wiki pages.
     Return new pages that user may want to upload."""
@@ -361,12 +393,13 @@ def run_main_sequence(pages, message, config, clients, cli_args):
         exit()
 
     # let's edit the pages
-    editor, editor_options = get_editor_and_options(config)
-    edition_dir, modified_files, all_files = edition_sequence(editor, editor_options, edition_dir, pages, clients)
+    editor_cmd = get_editor_command(config)
+    edition_dir, modified_files, all_files, deleted_page_filenames = edition_sequence(editor_cmd, edition_dir, pages, clients)
 
     # choose messages
     if modified_files:
-        messages = setdict_sequence(editor, editor_options, {fname: message for fname in modified_files}, 'files', 'discard from upload')
+        messages = {fname: DEFAULT_SUPPRESSION_MESSAGE if fname in deleted_page_filenames else DEFAULT_MESSAGE for fname in modified_files}
+        messages = setdict_sequence(editor_cmd, messages, 'files', 'discard from upload')
 
     # upload
     if not DRY_RUN and modified_files:
@@ -377,7 +410,7 @@ def run_main_sequence(pages, message, config, clients, cli_args):
     # cleanup
     new_files = tuple(map(os.path.basename, cleanup_known(edition_dir, modified_files, all_files, clients)))
     if new_files:
-        new_files = choice_sequence(editor, editor_options, new_files, 'files', 'discard from upload')
+        new_files = choice_sequence(editor_cmd, new_files, 'files', 'discard from upload')
     print(f"Done !  ({len(modified_files) or 'no'} files uploaded, {len(new_files) or 'no'} new files)")
     return new_files
 
@@ -389,22 +422,24 @@ def move_page(fullpagename:str, newfullname:str, clients, delete_source:bool, re
     # create the target page
     client_src, pagename = client_page_from_fullpagename(fullpagename, clients)
     client_trg, newname = client_page_from_fullpagename(newfullname, clients)
+    same_wiki = client_src == client_trg  # page is moved inside a wiki
     if try_lock(newname, client_trg):
         content = client_src.page(pagename)
-        r = client_trg.put_page(newname, content, f'moved from {fullpagename}')
+        verb = 'moved' if delete_source else 'copied'
+        r = client_trg.put_page(newname, content, f'{verb} from {pagename if same_wiki else fullpagename}')
         if r is not None:
             raise ValueError(f"Unexpected output for upload of page {newfullname}: {r}")
         # delete the source if asked to
         if delete_source:
             if try_lock(pagename, client_src):
                 content = REDIRECTION.format(newname=newfullname, pagename=fullpagename) if redirect else ''
-                client_src.put_page(pagename, content, f'moved to {newfullname}')
+                client_src.put_page(pagename, content, f'moved to {newname if same_wiki else newfullname}')
                 try_unlock(pagename, client_src)
             else:
-                print(f"Source {pagename} couldn't be deleted (locked)")
+                print(f"Source {pagename if same_wiki else fullpagename} couldn't be deleted (locked)")
         # unlock and quit
         if not try_unlock(newname, client_trg):
-            lprint(f"Couldn't unlock page {newfullname}.")
+            lprint(f"Couldn't unlock page {newname if same_wiki else newfullname}.")
         return True
 
 def list_named_pages(pagenames:[str], client) -> [str]:
@@ -512,8 +547,8 @@ def move_pages(pagenames:[str], target:str, config:str, clients, delete_source:b
         return
 
     # let user edit the moves
-    editor, editor_options = get_editor_and_options(config)
-    moves = setdict_sequence(editor, editor_options, moves, objname='renames' if delete_source else 'copies', action='cancel')
+    editor_cmd = get_editor_command(config)
+    moves = setdict_sequence(editor_cmd, moves, objname='renames' if delete_source else 'copies', action='cancel')
 
     # make the moves, fix backlinks
     unmoved_pages, unfixed_pages = set(), set()
@@ -537,13 +572,15 @@ def move_pages(pagenames:[str], target:str, config:str, clients, delete_source:b
     if unmoved_pages:
         comment[0] = f"Done!  ({len(unmoved_pages)} pages couldn't be moved"
         comment.append(f'the unfixed pages are saved in {save_list_into_tempfile(unfixed_pages)}')
-    else:
+    elif moves:
         comment[0] = "Done!  (all pages have been moved"
+    else:  # no moves operated
+        comment[0] = "Done!  (no page have been moved"
     if fix_backlinks:
         if unfixed_pages:
             comment[0] += f" and {len(unfixed_pages)} pages with backlinks couldn't be fixed)"
             comment.append(f' the unfixed pages are saved in {save_list_into_tempfile(unfixed_pages)}')
-        else:
+        elif moves:
             comment[0] += f" and all pages with backlinks were fixed"
     comment[0] += ")"
     for line in comment:
@@ -564,12 +601,18 @@ if __name__ == '__main__':
     # print(clients)
     # exit()
 
+    # handle aliases
+    pages = tuple(substitute_aliases(pages, args.config))
+    aliased_and_sanitized_page = lambda p: tuple(substitute_aliases([sanitize_input_pagename(p)], args.config))[0]
+
     if args.move_to:
         lprint('MOVE', args)
-        move_pages(pages, sanitize_input_pagename(args.move_to), args.config, clients, delete_source=True, redirect=args.redirect, fix_backlinks=args.fix_backlinks)
+        target = aliased_and_sanitized_page(args.move_to)
+        move_pages(pages, target, args.config, clients, delete_source=True, redirect=args.redirect, fix_backlinks=args.fix_backlinks)
     elif args.copy_to:
         lprint('COPY', args)
-        move_pages(pages, sanitize_input_pagename(args.move_to), args.config, clients, delete_source=False, redirect=args.redirect, fix_backlinks=args.fix_backlinks)
+        target = aliased_and_sanitized_page(args.copy_to)
+        move_pages(pages, target, args.config, clients, delete_source=False, redirect=args.redirect, fix_backlinks=args.fix_backlinks)
     else:  # just do page edition
         lprint('EDIT', args)
         edit_pages(pages, args.message, args.config, clients, args)
